package ru.bitel.bgbilling.modules.voice.dyn.mediator.md;
    
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.zip.ZipInputStream;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.apps.voice.accounting.mediation.VoiceRecordProcessor;
import ru.bitel.bgbilling.modules.voice.common.bean.VoiceDevice;
import ru.bitel.bgbilling.modules.voice.common.bean.VoiceDeviceType;
import ru.bitel.bgbilling.modules.voice.common.mediation.Mediator;
import ru.bitel.bgbilling.server.util.Setup;
import ru.bitel.common.ParameterMap;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.Utils;
    
public abstract class MDBaseMediator
    implements Mediator
{
    private static final Logger logger = LogManager.getLogger();

    private VoiceDevice device;

    @Override
    public Object init( Setup setup, int moduleId, VoiceDevice device, VoiceDeviceType deviceType, ParameterMap config )
        throws Exception
    {
        this.device = device;
        return null;
    }
    
    @Override
    public void readHourDataLog( VoiceRecordProcessor processor, Date hour )
        throws Exception
    {
        logger.debug( "readHourDataLog():" );

        LocalDateTime hourDateTime = TimeUtils.convertDateToLocalDateTime( hour );
        String hourDateTimeStr = hourDateTime != null ? DateTimeFormatter.ofPattern( "dd.MM.yyyy HH:mm" ).format( hourDateTime ) : "";
        String path = device.getLogPath();
        if ( logger.isDebugEnabled() )
        {
            logger.debug( "\thour = " + hourDateTimeStr );
            logger.debug( "\tpath = " + path );
        }

        String logPath = path + "/" + TimeUtils.format( hour, "yyyy/MM/dd_HH" ) + ".zip";
        if ( logger.isDebugEnabled() )
        {
            logger.debug( "logPath = " + logPath );
        }

        Path logFile = Paths.get( logPath );
        boolean exists = Files.exists( logFile );
        if ( logger.isDebugEnabled() )
        {
            logger.debug( "logFile exists => " + exists );
        }

        if ( !exists )
        {
            logger.warn( "log for " + hourDateTimeStr + " not exists" );
            return;
        }

        int columns = getColumns();
        try ( ZipInputStream zis = new ZipInputStream( new FileInputStream( logFile.toFile() ) ) )
        {
            zis.getNextEntry();
            BufferedReader reader = new BufferedReader( new InputStreamReader( zis ), 128 * 1024 );
            String line;
            while ( (line = reader.readLine()) != null )
            {
                String[] params = line.split( "\t" );
                if ( params.length == columns )
                {
                    logger.debug( "line = " + line );
                    processLine( processor, params );
                }
                else
                {
                    logger.warn( "Skip line: " + line );
                }
            }
        }
    }
    
    protected boolean skipLine( final String number )
    {
        if ( number.matches( "^46\\d{4,5}$" ) )
        {
            return true;
        }
        if ( number.matches( "^0\\d{1}$" ) )
        {
            return true;
        }
        return false;
    }

    protected void processLine( final VoiceRecordProcessor processor, final String[] params )
        throws InterruptedException
    {
        return;
        // String e164CallingStationId = params[1];
        // String e164CalledStationId = params[3];

        // e164CallingStationId = prepareNumber( e164CallingStationId );
        // e164CalledStationId = prepareNumber( e164CalledStationId );

        // final VoiceRecord record = processor.next();
        // record.trunkIncoming = params[0].substring( 1, 10 );
        // record.callingStationId = params[1];
        // record.calledStationId = params[3];
        // record.trunkOutgoing = params[2];
        // record.e164CallingStationId = e164CallingStationId;
        // record.e164CalledStationId = e164CalledStationId;
        // String dateTime = params[4] + " " + params[5];
        // record.sessionStart = TimeUtils.parseDate( dateTime, "dd-MM-yy
        // HH:mm:ss" );
        // record.connectionDuration = Utils.parseInt( params[6], 0 );
        // record.duration = Utils.parseInt( params[6], 0 );
        //
        // if ( logger.isDebugEnabled() )
        // {
        // logger.debug( "e164CallingStationId => " + e164CallingStationId + ";
        // e164CalledStationId = " + e164CalledStationId + "; trunkIncoming = "
        // + record.trunkIncoming );
        // }
    }

    protected int getColumns()
    {
        return 1;
    }

    @Override
    public void getLogExists( Date month, int[] data )
    {
        String path = device.getLogPath();

        if ( logger.isDebugEnabled() )
        {
            logger.debug( "getLogExists: month = " + month + "; path = " + path );
        }

        Path rootDirPath = Paths.get( path );
        if ( Files.notExists( rootDirPath ) )
        {
            if ( logger.isDebugEnabled() )
            {
                logger.debug( "rootDirPath = " + rootDirPath + " not exists" );
            }
            return;
        }

        LocalDate monthLocalDate = TimeUtils.convertDateToLocalDate( month );

        String mm = String.format( "%02d", monthLocalDate.getMonthValue() );
        String yy = String.format( "%04d", monthLocalDate.getYear() );

        if ( logger.isDebugEnabled() )
        {
            logger.debug( "monthLocalDate = " + monthLocalDate + "; mm = " + mm + "; yy = " + yy );
        }

        Path monthDirPath = Paths.get( rootDirPath.toString(), yy, mm );
        if ( Files.notExists( monthDirPath ) )
        {
            if ( logger.isDebugEnabled() )
            {
                logger.debug( "monthDirPath = " + monthDirPath + " not exists" );
            }
            return;
        }

        if ( logger.isDebugEnabled() )
        {
            logger.debug( "monthDirPath = " + monthDirPath );
        }

        for ( File cdrDayHour : monthDirPath.toFile().listFiles() )
        {
            if ( cdrDayHour.isFile() )
            {
                String fileName = cdrDayHour.getName().toString();
                int day = Utils.parseInt( fileName.substring( 0, 2 ) );
                int hour = Utils.parseInt( fileName.substring( 3, 5 ) );
                // logger.debug( "fileName = " + fileName + " day = " + day + "
                // hour = " + hour );
                data[day - 1] |= (1 << hour);
            }
        }
    }
}
